package de.ozgcloud.eingang.fim;

import static java.util.Collections.*;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.xta.RepresentationsAttachmentsPair;
import de.ozgcloud.eingang.xta.XtaMessageHandler;
import de.ozgcloud.eingang.xta.XtaMessageMapper;
import de.ozgcloud.eingang.xta.XtaMessageMetaData;

@Component
public class FimXtaMessageHandler implements XtaMessageHandler {
	static final String ALL_FIM_MESSAGES_PREFIX = "fim.";

	static final String FIM_FILE_GROUP_NAME = "fim_files";

	@Autowired
	private XtaMessageMapper xtaMessageMapper;

	@Autowired
	private VorgangNummerSupplier vorgangNummerSupplier;

	@Override
	public FormData getFormData(XtaMessageMetaData xtaMessageMetaData, List<IncomingFile> incomingFiles) {
		return xtaMessageMapper.toFormData(
				getRepresentationsAttachmentsPair(incomingFiles),
				xtaMessageMetaData,
				vorgangNummerSupplier
		);
	}

	RepresentationsAttachmentsPair getRepresentationsAttachmentsPair(List<IncomingFile> incomingFiles) {
		return RepresentationsAttachmentsPair.builder()
				.representations(emptyList())
				.attachment(IncomingFileGroup.builder()
						.name(FIM_FILE_GROUP_NAME)
						.files(incomingFiles)
						.build())
				.build();
	}

	@Override
	public boolean isResponsible(XtaMessageMetaData xtaMessageMetaData) {
		return StringUtils.startsWith(xtaMessageMetaData.getMessageType(), ALL_FIM_MESSAGES_PREFIX);
	}
}
