package de.ozgcloud.eingang.xdomea;

import java.util.List;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.xta.RepresentationsAttachmentsPair;
import de.ozgcloud.eingang.xta.XtaMessageHandler;
import de.ozgcloud.eingang.xta.XtaMessageMapper;
import de.ozgcloud.eingang.xta.XtaMessageMetaData;

@Component
public class XdomeaXtaMessageHandler implements XtaMessageHandler {
	static final String XDOMEA_0201_MESSAGE_TYPE = "Geschaeftsgang.Geschaeftsgang.0201";

	@Autowired
	private XtaMessageMapper xtaMessageMapper;

	@Autowired
	private XdomeaMessageDataMapper xdomeaMessageDataMapper;

	@Autowired
	private VorgangNummerSupplier vorgangNummerSupplier;

	@Override
	public FormData getFormData(XtaMessageMetaData xtaMessageMetaData, List<IncomingFile> incomingFiles) {
		return xtaMessageMapper.toFormData(
				getRepresentationsAttachmentsPair(incomingFiles),
				xtaMessageMetaData,
				vorgangNummerSupplier
		);
	}

	RepresentationsAttachmentsPair getRepresentationsAttachmentsPair(List<IncomingFile> incomingFiles) {
		var xdomeaMessageData = xdomeaMessageDataMapper.mapIncomingFilesToXdomeaMessageData(incomingFiles);
		return RepresentationsAttachmentsPair.builder()
				.representations(getRepresentations(xdomeaMessageData))
				.attachments(xdomeaMessageData.attachments())
				.build();
	}

	private List<IncomingFile> getRepresentations(XdomeaMessageData xdomeaMessageData) {
		return Stream.concat(
				Stream.of(xdomeaMessageData.metadataFile()),
				xdomeaMessageData.representations().stream()
		).toList();
	}

	@Override
	public boolean isResponsible(XtaMessageMetaData xtaMessageMetaData) {
		return StringUtils.equals(xtaMessageMetaData.getMessageType(), XDOMEA_0201_MESSAGE_TYPE);
	}
}
