package de.ozgcloud.eingang.xta;

import org.mapstruct.Context;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper;

@Mapper(imports = FilesMapperHelper.class)
public interface XtaMessageMapper {
	int VORGANG_NUMMER_SUFFIX_LENGTH = 4;

	@Mapping(target = "antragsteller", ignore = true)
	@Mapping(target = "formData", ignore = true)
	@Mapping(target = "id", ignore = true)
	@Mapping(target = "zustaendigeStelle", ignore = true)
	@Mapping(target = "header", source = "metaData")
	@Mapping(target = "numberOfAttachments", expression = "java(FilesMapperHelper.countAttachedFiles(representationsAttachmentsPair.attachments()))")
	@Mapping(target = "numberOfRepresentations", expression = "java(representationsAttachmentsPair.representations().size())")
	@Mapping(target = "representation", ignore = true)
	@Mapping(target = "attachment", ignore = true)
	FormData toFormData(
			RepresentationsAttachmentsPair representationsAttachmentsPair,
			XtaMessageMetaData metaData,
			@Context VorgangNummerSupplier vorgangNummerSupplier
	);

	@Mapping(target = "formId", source = "messageType")
	@Mapping(target = "requestId", source = "messageId")
	@Mapping(target = "vorgangNummer", expression = "java(vorgangNummerSupplier.get(VORGANG_NUMMER_SUFFIX_LENGTH))")
	@Mapping(target = "serviceKonto", ignore = true)
	@Mapping(target = "createdAt", source = "origin")
	@Mapping(target = "sender", constant = "XTA")
	@Mapping(target = "formName", ignore = true)
	@Mapping(target = "formEngineName", ignore = true)
	FormHeader formHeaderFromMetaData(XtaMessageMetaData metaData, @Context VorgangNummerSupplier vorgangNummerSupplier);

	default String fromId(XtaMessageId id) {
		return id.toString();
	}
}
