package de.ozgcloud.eingang.xta;

import java.util.List;
import java.util.Optional;
import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.eingang.common.formdata.FormData;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class XtaService {
	@Autowired
	private XtaRemoteService remoteService;

	@Autowired
	private XtaIncomingFilesMapper xtaIncomingFilesMapper;

	@Autowired
	private List<XtaMessageHandler> xtaMessageHandlers;

	public Stream<FormData> getMessages() {
		return createXtaMessageStream().map(this::mapToFormData).flatMap(Optional::stream);
	}

	Optional<FormData> mapToFormData(XtaMessageMetaData messageMetaData) {
		return getResponsibleHandler(messageMetaData).map(handler -> getFormData(messageMetaData, handler));
	}

	Stream<XtaMessageMetaData> createXtaMessageStream() {
		var iterator = new XtaMessageMetadataRemoteIterator(remoteService);
		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 0), false);
	}

	Optional<XtaMessageHandler> getResponsibleHandler(XtaMessageMetaData messageMetaData) {
		var handlerOptional = xtaMessageHandlers.stream()
				.filter(handler -> handler.isResponsible(messageMetaData))
				.findFirst();
		if (handlerOptional.isEmpty()) {
			logWarningForUnsupportedMessageType(messageMetaData.getMessageType());
		}
		return handlerOptional;
	}

	void logWarningForUnsupportedMessageType(String messageType) {
		LOG.warn("Ignoring XTA-Message of type '{}'.", messageType);
	}

	FormData getFormData(@NonNull XtaMessageMetaData metaData, XtaMessageHandler handler) {
		var msg = remoteService.getMessage(metaData.getMessageId());
		var incomingFiles = xtaIncomingFilesMapper.toIncomingFiles(msg.getMessageFiles());
		return handler.getFormData(metaData, incomingFiles);
	}

	public void acknowledgeReceive(@NonNull XtaMessageId messageId) {
		remoteService.close(messageId);
	}
}
