package de.ozgcloud.eingang.fim;

import static de.ozgcloud.eingang.fim.FimXtaMessageHandler.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.xta.RepresentationsAttachmentsPair;
import de.ozgcloud.eingang.xta.XtaMessageMapper;
import de.ozgcloud.eingang.xta.XtaMessageMetaData;

class FimXtaMessageHandlerTest {

	@Spy
	@InjectMocks
	private FimXtaMessageHandler handler;

	@Mock
	private XtaMessageMapper xtaMessageMapper;

	@Mock
	private VorgangNummerSupplier vorgangNummerSupplier;

	@Mock
	private XtaMessageMetaData xtaMessageMetaData;

	@Mock
	private RepresentationsAttachmentsPair representationsAttachmentsPair;

	@DisplayName("get form data")
	@Nested
	class TestGetFormData {

		@Mock
		private List<IncomingFile> incomingFiles;

		@Mock
		private FormData formData;

		@DisplayName("should return form data")
		@Test
		void shouldReturnFormData() {
			doReturn(representationsAttachmentsPair).when(handler).getRepresentationsAttachmentsPair(incomingFiles);
			when(xtaMessageMapper.toFormData(representationsAttachmentsPair, xtaMessageMetaData, vorgangNummerSupplier)).thenReturn(formData);

			var result = handler.getFormData(xtaMessageMetaData, incomingFiles);

			assertThat(result).isEqualTo(formData);
		}
	}

	@DisplayName("get representations attachments pair")
	@Nested
	class TestGetRepresentationsAttachmentsPair {

		@Mock
		private IncomingFile incomingFile1;

		@Mock
		private IncomingFile incomingFile2;

		private List<IncomingFile> incomingFiles;

		@BeforeEach
		void mock() {
			incomingFiles = List.of(incomingFile1, incomingFile2);
		}

		@DisplayName("should return empty representations")
		@Test
		void shouldReturnRepresentations() {
			var result = handler.getRepresentationsAttachmentsPair(incomingFiles);

			assertThat(result.representations()).isEmpty();
		}

		@DisplayName("should return one attachment group")
		@Test
		void shouldReturnEmptyAttachments() {
			var result = handler.getRepresentationsAttachmentsPair(incomingFiles);

			assertThat(result.attachments()).hasSize(1);
		}

		@DisplayName("should have name")
		@Test
		void shouldHaveName() {
			var group = firstGroup();

			assertThat(group.getName()).isEqualTo(FIM_FILE_GROUP_NAME);
		}

		@DisplayName("should have attachment files")
		@Test
		void shouldHaveAttachmentFiles() {
			var result = firstGroup();

			assertThat(result.getFiles()).containsExactly(incomingFile1, incomingFile2);
		}

		private IncomingFileGroup firstGroup() {
			var result = handler.getRepresentationsAttachmentsPair(incomingFiles);
			return result.attachments().getFirst();
		}
	}

	@DisplayName("is responsible")
	@Nested
	class TestIsResponsible {

		@DisplayName("should return true")
		@Test
		void shouldReturnTrue() {
			when(xtaMessageMetaData.getMessageType()).thenReturn("fim.123");

			var result = handler.isResponsible(xtaMessageMetaData);

			assertThat(result).isTrue();
		}

		@DisplayName("should return false")
		@Test
		void shouldReturnFalse() {
			when(xtaMessageMetaData.getMessageType()).thenReturn("xdomea.123");

			var result = handler.isResponsible(xtaMessageMetaData);

			assertThat(result).isFalse();
		}
	}
}
