package de.ozgcloud.eingang.xdomea;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.xta.RepresentationsAttachmentsPair;
import de.ozgcloud.eingang.xta.XtaMessageMapper;
import de.ozgcloud.eingang.xta.XtaMessageMetaData;

class XdomeaXtaMessageHandlerTest {

	@Spy
	@InjectMocks
	private XdomeaXtaMessageHandler handler;

	@Mock
	private XtaMessageMapper xtaMessageMapper;

	@Mock
	private XdomeaMessageDataMapper xdomeaMessageDataMapper;

	@Mock
	private VorgangNummerSupplier vorgangNummerSupplier;

	@Mock
	private XtaMessageMetaData xtaMessageMetaData;

	@DisplayName("get form data")
	@Nested
	class TestGetFormData {

		@Mock
		private List<IncomingFile> incomingFiles;

		@Mock
		private RepresentationsAttachmentsPair representationsAttachmentsPair;

		@Mock
		private FormData formData;

		@DisplayName("should map to form data")
		@Test
		void shouldMapToFormData() {
			doReturn(representationsAttachmentsPair).when(handler).getRepresentationsAttachmentsPair(incomingFiles);
			when(xtaMessageMapper.toFormData(
					representationsAttachmentsPair,
					xtaMessageMetaData,
					vorgangNummerSupplier
			)).thenReturn(formData);

			var result = handler.getFormData(xtaMessageMetaData, incomingFiles);

			assertThat(result).isEqualTo(formData);
		}
	}

	@DisplayName("get representations attachments pair")
	@Nested
	class TestGetRepresentationsAttachmentsPair {
		@Mock
		private List<IncomingFile> incomingFiles;

		private XdomeaMessageData xdomeaMessageData;

		@Mock
		private IncomingFile incomingFile1;

		@Mock
		private IncomingFile incomingFile2;

		@BeforeEach
		void mock() {
			xdomeaMessageData = XdomeaMessageDataTestFactory.createBuilder()
					.representations(List.of(incomingFile1, incomingFile2))
					.build();
			when(xdomeaMessageDataMapper.mapIncomingFilesToXdomeaMessageData(incomingFiles)).thenReturn(xdomeaMessageData);
		}

		@DisplayName("should return representations")
		@Test
		void shouldReturnRepresentations() {
			var result = handler.getRepresentationsAttachmentsPair(incomingFiles);

			assertThat(result.representations()).containsExactly(
					xdomeaMessageData.metadataFile(),
					incomingFile1,
					incomingFile2
			);
		}

		@DisplayName("should return attachments")
		@Test
		void shouldReturnAttachments() {
			var result = handler.getRepresentationsAttachmentsPair(incomingFiles);

			assertThat(result.attachments()).isEqualTo(xdomeaMessageData.attachments());
		}
	}

	@DisplayName("is responsible")
	@Nested
	class TestIsResponsible {
		@DisplayName("should return true for 0201")
		@Test
		void shouldReturnTrueFor0201() {
			when(xtaMessageMetaData.getMessageType()).thenReturn(XdomeaXtaMessageHandler.XDOMEA_0201_MESSAGE_TYPE);

			var result = handler.isResponsible(xtaMessageMetaData);

			assertThat(result).isTrue();
		}

		@DisplayName("should return false for other")
		@Test
		void shouldReturnFalseForOther() {
			when(xtaMessageMetaData.getMessageType()).thenReturn("other");

			var result = handler.isResponsible(xtaMessageMetaData);

			assertThat(result).isFalse();
		}
	}
}
