package de.ozgcloud.eingang.xta;

import static de.ozgcloud.eingang.xta.RepresentationsAttachmentsPairTestFactory.*;
import static de.ozgcloud.eingang.xta.XtaMessageMapper.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;

class XtaMessageMapperTest {

	@Spy
	@InjectMocks
	private XtaMessageMapper mapper = Mappers.getMapper(XtaMessageMapper.class);

	@Mock
	private VorgangNummerSupplier vorgangNummerSupplier;

	@Nested
	class TestToFormData {

		private XtaMessageMetaData xtaMessageMetaData;
		private RepresentationsAttachmentsPair representationsAttachmentsPair;

		@BeforeEach
		void mock() {
			xtaMessageMetaData = XtaMessageMetaDataTestFactory.create();
			representationsAttachmentsPair = RepresentationsAttachmentsPairTestFactory.create();
			when(vorgangNummerSupplier.get(VORGANG_NUMMER_SUFFIX_LENGTH)).thenReturn(FormHeaderTestFactory.VORGANGNUMMER);
		}

		@Test
		void shouldMapRepresentations() {
			var formData = doMapping();

			assertThat(formData.getRepresentations()).isEqualTo(PAIR_REPRESENTATIONS);
		}

		@Test
		void shouldSetNumberOfAttachments() {
			var formData = doMapping();

			assertThat(formData.getNumberOfAttachments()).isEqualTo(PAIR_ATTACHMENTS.size());
		}

		@Test
		void shouldSetNumberOfRepresentations() {
			var formData = doMapping();

			assertThat(formData.getNumberOfRepresentations()).isEqualTo(PAIR_REPRESENTATIONS.size());
		}

		@Test
		void shouldMapAttachments() {
			var formData = doMapping();

			assertThat(formData.getAttachments()).isEqualTo(PAIR_ATTACHMENTS);
		}

		@Test
		void shouldMapVorgangNummer() {
			var formData = doMapping();

			assertThat(formData.getHeader().getVorgangNummer()).isEqualTo(FormHeaderTestFactory.VORGANGNUMMER);
		}

		private FormData doMapping() {
			return mapper.toFormData(representationsAttachmentsPair, xtaMessageMetaData, vorgangNummerSupplier);
		}
	}
}
